import { wxApply } from "../wxGame/wxApply";
import { Global } from "../Global";
import PopShareUI from "./PopShareUI";

const { ccclass, property } = cc._decorator;

let self: ShopUI
@ccclass
export default class ShopUI extends cc.Component {
    private btn_sharePrize: cc.Button;     //分享的金币
    private btn_videoPrize: cc.Node;      //看视频的金币


    private btn_randomBallBuy: cc.Node   //随机购买篮球按钮
    private btn_randomLineBuy: cc.Node   //随机购买篮球按钮

    @property(cc.SpriteFrame)
    private ballSpriteArray = [];       //全部篮球精灵

    @property(cc.SpriteFrame)
    private lineSpriteArray = [];       //全部篮球精灵

    private ballShopNode: cc.Node;       //篮球商店节点
    private LineShopNode: cc.Node;       //线条商店节点

    private ballShopArray = [];         //篮球商店节点里的子物体
    private LineShopArray = [];         //线条商店节点里的子物体

    private curSelectBall;              //当前选择的篮球
    private curSelectLine;              //当前选择的线条

    private buyBallGold: cc.Label;           //随机购买篮球需要的价格
    private buyLineGold: cc.Label;           //随机购买线条需要的价格

    private btn_ballShop: cc.Node;
    private btn_lineShop: cc.Node;        

    @property(cc.Node)
    private tip_buyShop: cc.Node = null;    

    @property(PopShareUI)
    private popShareUI: PopShareUI = null;   

    private isCanBuy = true;

    onLoad() {
        self = this;
        self.btn_sharePrize = self.node.getChildByName("btn_sharePrize").getComponent(cc.Button);
        self.btn_videoPrize = self.node.getChildByName("btn_videoPrize");


        self.btn_randomBallBuy = self.node.getChildByName("btn_randomBallBuy");
        self.btn_randomLineBuy = self.node.getChildByName("btn_randomLineBuy");


        self.buyBallGold = self.btn_randomBallBuy.getChildByName("buyBallGold").getComponent(cc.Label);
        self.buyLineGold = self.btn_randomLineBuy.getChildByName("buyLineGold").getComponent(cc.Label);

        self.ballShopNode = self.node.getChildByName("ballShopNode");
        self.LineShopNode = self.node.getChildByName("LineShopNode");

        self.ballShopArray = self.ballShopNode.children;
        self.LineShopArray = self.LineShopNode.children;

        self.btn_ballShop = self.node.getChildByName("shopGroup").getChildByName("btn_ballShop");
        self.btn_lineShop = self.node.getChildByName("shopGroup").getChildByName("btn_lineShop");

        for (let i = 0; i < self.ballShopArray.length; i++) {
            self.ballShopArray[i].on(cc.Node.EventType.TOUCH_START, self.selectBall, self);
        }

        for (let i = 0; i < self.LineShopArray.length; i++) {
            self.LineShopArray[i].on(cc.Node.EventType.TOUCH_START, self.selectLine, self);
        }

        self.btn_videoPrize.on(cc.Node.EventType.TOUCH_END, self.videoPrize, self);
        self.btn_randomBallBuy.on(cc.Node.EventType.TOUCH_END, self.randomBallBuy, self);
        self.btn_randomLineBuy.on(cc.Node.EventType.TOUCH_END, self.randomLineBuy, self);

        self.btn_ballShop.on(cc.Node.EventType.TOUCH_START, self.openBallShop, self);
        self.btn_lineShop.on(cc.Node.EventType.TOUCH_START, self.openLineShop, self);

        cc.director.on("closeShareGold", self.closeShareGold, self);
    }

    start() {
        self.openBallShop();
        Global.judgeShareDayMax();


        if (Global.getConfig2) {
            if (Global.getConfig2.shareGoldSwtich == 0) {
                self.btn_sharePrize.node.active = false;
            }

            if (Global.getConfig2.videoGoldSwitch == 0) {
                self.btn_videoPrize.active = false;
            }
        }

        if(Global.USER_INFO.newPlayer){     //如果是新玩家
            self.tip_buyShop.active = true;
            self.tip_buyShop.runAction(cc.repeatForever(
                cc.sequence(
                    cc.moveBy(0.3, -20, 20),
                    cc.moveBy(0.3, 20, -20)
                )
            ))
        }
        else{
            self.tip_buyShop.active = false;
        }
    }

    onEnable() {
        self.setBallShopInfo();
        self.setLineShopInfo();
    }

    onDisable() {
        cc.director.emit("updateWuPin");
    }

    private openBallShop() {
        self.LineShopNode.active = false;
        self.btn_randomLineBuy.active = false;

        self.ballShopNode.active = true;
        self.btn_randomBallBuy.active = true;
    }

    private openLineShop() {
        self.ballShopNode.active = false;
        self.btn_randomBallBuy.active = false;

        self.LineShopNode.active = true;
        self.btn_randomLineBuy.active = true;
    }

    //开始随机购买篮球物品
    private randomBallBuy() {
        if(!this.isCanBuy){
            return;
        }
        else{
            this.isCanBuy = false;
            setTimeout(()=>{
                this.isCanBuy = true;
            }, 5000);
        }

        self.tip_buyShop.active = false;

        let useGold = parseInt(self.buyBallGold.string);
        if (!Global.useGold(useGold)) {
            this.isCanBuy = true;
            return;
        }

        let ownBall = Global.ownBall;
        let tempArray = [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15];
        for (let i = 0; i < ownBall.length; i++) {
            let tempNumber = ownBall[i];
            for (let j = 0; j < tempArray.length; j++) {
                if (tempArray[j] == tempNumber) {
                    tempArray.splice(j, 1);
                    break;
                }
            }
        }

        let ballShopArray = self.ballShopArray;
        let randomRange = tempArray.length;
        var randomNumberUp = 0;

        for (let i = 0; i < 20; i++) {
            let randomNumber = Math.floor(Math.random() * randomRange)
            if (randomNumberUp != randomNumber) {
                randomNumberUp = randomNumber;
            }
            else {
                randomNumber = Math.floor(Math.random() * randomRange);
            }

            let tempBall = ballShopArray[tempArray[randomNumber]];
            if (tempBall) {
                if (i != 19) {
                    self.scheduleOnce(() => {
                        tempBall.getChildByName("randomFlice").runAction(cc.sequence(
                            cc.fadeIn(0.075),
                            cc.fadeOut(0.075)
                        ))

                        tempBall.getChildByName("mySprite").runAction(cc.sequence(
                            cc.scaleTo(0.075, 1.2, 1.2),
                            cc.scaleTo(0.075, 1, 1)
                        ))
                    }, i * 0.15)
                }
                else {
                    self.scheduleOnce(() => {
                        self.getBall(tempArray[randomNumber]);
                        for (let j = 0; j < 10; j++) {
                            self.scheduleOnce(() => {
                                tempBall.getChildByName("selectFlicker").runAction(cc.sequence(
                                    cc.fadeIn(0.05),
                                    cc.fadeOut(0.05)
                                ))
                            }, j * 0.1)
                        }
                    }, i * 0.15)
                }
            }
        }
    }

    //开始随机购买线条物品
    private randomLineBuy() {
        if(!this.isCanBuy){
            return;
        }
        else{
            this.isCanBuy = false;
            setTimeout(()=>{
                this.isCanBuy = true;
            }, 5000);
        }
        

        self.tip_buyShop.active = false;

        let useGold = parseInt(self.buyLineGold.string);
        if (!Global.useGold(useGold)) {
            this.isCanBuy = true;
            return;
        }

        let ownLine = Global.ownLine;
        let tempArray = [0, 1, 2, 3, 4, 5, 6];
        for (let i = 0; i < ownLine.length; i++) {
            let tempNumber = ownLine[i];
            for (let j = 0; j < tempArray.length; j++) {
                if (tempArray[j] == tempNumber) {
                    tempArray.splice(j, 1);
                    break;
                }
            }
        }

        let LineShopArray = self.LineShopArray;
        let randomRange = tempArray.length;
        var randomNumberUp = 0;

        for (let i = 0; i < 20; i++) {
            let randomNumber = Math.floor(Math.random() * randomRange)
            if (randomNumberUp != randomNumber) {
                randomNumberUp = randomNumber;
            }
            else {
                randomNumber = Math.floor(Math.random() * randomRange);
            }

            let tempBall = LineShopArray[tempArray[randomNumber]];
            if (tempBall) {
                if (i != 19) {
                    self.scheduleOnce(() => {
                        tempBall.getChildByName("randomFlice").runAction(cc.sequence(
                            cc.fadeIn(0.075),
                            cc.fadeOut(0.075)
                        ))

                        tempBall.getChildByName("mySprite").runAction(cc.sequence(
                            cc.scaleTo(0.075, 1.2, 1.2),
                            cc.scaleTo(0.075, 1, 1)
                        ))
                    }, i * 0.15)
                }
                else {
                    self.scheduleOnce(() => {
                        self.getLine(tempArray[randomNumber]);
                        for (let j = 0; j < 10; j++) {
                            self.scheduleOnce(() => {
                                tempBall.getChildByName("selectFlicker").runAction(cc.sequence(
                                    cc.fadeIn(0.05),
                                    cc.fadeOut(0.05)
                                ))
                            }, j * 0.1)
                        }
                    }, i * 0.15)
                }
            }
        }
    }

    private setBallShopInfo() {
        let ownBall = Global.ownBall;
        for (let i = 0; i < ownBall.length; i++) {
            let ballId = ownBall[i];
            self.ballShopArray[ballId].getChildByName("mySprite").getComponent(cc.Sprite).spriteFrame = self.ballSpriteArray[ballId];
        }

        self.curSelectBall = self.ballShopArray[Global.USER_INFO.selectBall];
        self.ballShopArray[Global.USER_INFO.selectBall].getChildByName("select").active = true;

        self.setbuyBallGold();
    }
    private setLineShopInfo() {
        let ownLine = Global.ownLine;
        for (let i = 0; i < ownLine.length; i++) {
            let lineId = ownLine[i];
            self.LineShopArray[lineId].getChildByName("mySprite").getComponent(cc.Sprite).spriteFrame = self.lineSpriteArray[lineId];
        }

        self.curSelectLine = self.LineShopArray[Global.USER_INFO.selectLine];
        self.LineShopArray[Global.USER_INFO.selectLine].getChildByName("select").active = true;

        self.setbuyLineGold();
    }

    //-------------------------------------------------------------
    private sharePrize() {
        wxApply.shareAppMessage(3);        
    }

    private videoPrize() {
        Global.createRewardedVideoAd("adunit-b38863c17f25ef30", self.prizeGold);
    }

    private prizeGold(){
        Global.addGold(25);
   }

    private selectBall(obj) {
        var target = obj.currentTarget;
        var targetName = parseInt(target.name);

        let ownBall = Global.ownBall;
        for (let i = 0; i < ownBall.length; i++) {
            if (ownBall[i] == targetName) {
                self.setCurBall(targetName);
                return;
            }
        }
    }

    private selectLine(obj) {
        var target = obj.currentTarget;
        var targetName = parseInt(target.name);

        let ownLine = Global.ownLine;
        for (let i = 0; i < ownLine.length; i++) {
            if (ownLine[i] == targetName) {
                self.setCurLine(targetName);
                return;
            }
        }
    }

    private getBall(ballId) {
        Global.ownBall.push(ballId);
        self.ballShopArray[ballId].getChildByName("mySprite").getComponent(cc.Sprite).spriteFrame = self.ballSpriteArray[ballId];
        self.setCurBall(ballId);
        self.setbuyBallGold();        

        if(Global.USER_INFO.newPlayer){
            Global.USER_INFO.newPlayer = false;
            cc.director.emit("shopping");
        }

        Global.saveNativeData();
        Global.saveShopData();
        

        if (Global.getConfig2) {
            if (Global.getConfig2.shareGoldSwtich != 0) {
                self.popShareUI.node.active = true;
                self.popShareUI.setMySprite(1);
            }
        }
    }

    private getLine(lineId) {
        Global.ownLine.push(lineId);
        self.LineShopArray[lineId].getChildByName("mySprite").getComponent(cc.Sprite).spriteFrame = self.lineSpriteArray[lineId];
        self.setCurLine(lineId);
        self.setbuyLineGold();        

        if(Global.USER_INFO.newPlayer){
            Global.USER_INFO.newPlayer = false;
            cc.director.emit("shopping");
        }

        Global.saveNativeData();
        Global.saveShopData();

        if (Global.getConfig2) {
            if (Global.getConfig2.shareGoldSwtich != 0) {
                self.popShareUI.node.active = true;
                self.popShareUI.setMySprite(2);
            }
        }
    }

    private setbuyBallGold() {
        let buyBallGold = Global.ownBall.length * 100;
        self.buyBallGold.string = buyBallGold + "";
    }

    private setbuyLineGold() {
        let buyLineGold = Global.ownLine.length * 100;
        self.buyLineGold.string = buyLineGold + "";
    }

    private setCurBall(id: number) {
        self.curSelectBall.getChildByName("select").active = false;
        self.curSelectBall = self.ballShopArray[id];
        self.curSelectBall.getChildByName("select").active = true;
        Global.USER_INFO.selectBall = id;
    }
    private setCurLine(id: number) {
        self.curSelectLine.getChildByName("select").active = false;
        self.curSelectLine = self.LineShopArray[id];
        self.curSelectLine.getChildByName("select").active = true;

        Global.USER_INFO.selectLine = id;

        switch (id) {
            case 0:     //灰色
                Global.USER_INFO.curLineColor = "#C4C4C4";
                break;
            case 1:    //蓝色
                Global.USER_INFO.curLineColor = "#1DA9FF";
                break;
            case 2:    //绿色
                Global.USER_INFO.curLineColor = "#53C10C";
                break;
            case 3:    //粉色
                Global.USER_INFO.curLineColor = "#FF527B";
                break;
            case 4:    //紫色
                Global.USER_INFO.curLineColor = "#BE57FF";
                break;
            case 5:    //红色
                Global.USER_INFO.curLineColor = "#FF2B2B";
                break;
            case 6:    //橙色
                Global.USER_INFO.curLineColor = "#FF8401";
                break;
            default:
                break;
        }
        console.log("颜色", Global.USER_INFO.curLineColor);
    }
    //-------------------------------------------------------------
    private closeShareGold() {
        console.log("禁用商店分享按钮");
        self.btn_sharePrize.getComponent(cc.Button).interactable = false;
    }


}
