import { Global } from "../Global";
import BasketLogic from "./BasketLogic";
import { Const } from "../Const";
import { Message } from "../Message";


const { ccclass, property } = cc._decorator;

@ccclass
export default class BallConLogic extends cc.Component {

    private physics: cc.RigidBody;  //球的物理引擎    

    private coliderCount = 0;

    private agreeCount = 0;  //是否同意游戏结束    

    private isOver = false;

    onLoad() {
        var self = this;

        cc.director.on("updateWuPin", self.updateWuPin, self);
        cc.director.on("updateBall", self.updateBall, self);

        self.physics = this.getComponent(cc.RigidBody);

        self.physics.enabledContactListener = true;

        if(!Global.isGameStart){
            this.updateWuPin();
        }        
    }

    onEnable() {

    }

    onDisable() {

    }

    private addEvent() {
        var self = this;
        cc.director.on("ballInit", self.ballInit, self);        
        cc.director.on("openOverTimer", self.openOverTimer, self);        
    }

    private openOverTimer() {
        this.schedule(this.checkStop, 1);
        this.scheduleOnce(() => {
            cc.director.emit("gameOver");
        }, 20);
    }

    private checkStop() {
        let pos = this.physics.linearVelocity;
        if (pos.x <= 10 && pos.y <= 10 && this.agreeCount == 5) {
            cc.director.emit("gameOver");
            this.unschedule(this.checkStop);
        }
        else if (pos.x <= 20 && pos.y <= 20) {
            this.agreeCount++;
        }
        else {
            this.agreeCount = 0;
        }
    }

    private ballInit() {
        this.physics.gravityScale = 1;
        this.physics.angularVelocity = -300;
    }

    public backHome() {
        // this.node.active = false;
        // this.node.parent = Global.ballParent;
        // this.node.group = "ball";                
        // this.initData(); 
        this.node.destroy();
    }

    private setMoveInfo(x, y) {
        this.physics.linearVelocity = cc.v2(x, y);
    }

    private updateWuPin() {
        console.log("读取篮球图片");
        cc.loader.loadRes(`Image/Ball/ball_${Global.USER_INFO.selectBall}`, cc.SpriteFrame, function (err, data) {
            if (err) {
                console.error("读取篮球图片失败");
            }
            else {
                cc.director.emit("updateBall", {
                    mySprite: data
                })
            }
        })
    }

    private updateBall(data) {
        var mySprite = data.getUserData().mySprite;
        this.getComponent(cc.Sprite).spriteFrame = mySprite;
    }

    private changMyGroup() {
        this.node.active = false;

        this.physics.gravityScale = 0;
        this.physics.linearVelocity = cc.v2(0, 0);
        //this.physics.angularVelocity = 0;
        this.physics.linearDamping = 0;

        this.node.group = "ghost";
        this.node.active = true;

        this.physics.gravityScale = 1;
    }

    private initData() {
        this.physics.gravityScale = 0;
        this.physics.linearVelocity = cc.v2(0, 0);
        this.physics.angularVelocity = 0;
        this.physics.linearDamping = 0;
        this.coliderCount = 0;
    }

    private onBeginContact(contact, selfCollider, otherCollider) {
        if (otherCollider.tag == Const.bascktFrameTag) {        //碰到篮框
            Global.playMusic("ball_collide");
            this.coliderCount++;
        }
        else if (otherCollider.tag == Const.basketCenterTag) {        //进球      
            if (this.isOver) {
                return;
            }

            this.isOver = true;

            let basket: BasketLogic = otherCollider.getComponent(BasketLogic);
            basket.showHideEffect();

            if (this.coliderCount < 1) {
                cc.director.emit("addScore", {
                    score: 200,
                    pos: otherCollider.node.getPosition(),
                    dir: otherCollider.node.scaleX,
                    type: 1
                })
                Global.addSocre(200);
            }
            else {
                cc.director.emit("addScore", {
                    score: 100,
                    pos: otherCollider.node.getPosition(),
                    dir: otherCollider.node.scaleX,
                    type: 1
                })
                Global.addSocre(100);
            }

            this.backHome();
            Message.sendIsVictory();
        }
        else if (otherCollider.tag == Const.BottomWallTag) {        //掉下去                 
            this.scheduleOnce(() => {
                this.backHome();
                cc.director.emit("gameOver");
            }, 0.2);
        }
        else if (otherCollider.tag == Const.elasticityTag) {        //碰到弹力块
            let selfNode = selfCollider.node;
            let otherNode = otherCollider.node;
            let padX = selfNode.x - otherNode.x;
            let padY = selfNode.y - otherNode.y;

            this.setMoveInfo(padX * 10, padY * 10);

            otherCollider.node.runAction(cc.sequence(
                cc.scaleTo(0.1, 0.8, 0.8),
                cc.scaleTo(0.1, 1, 1)
            ))
        }
        else if (otherCollider.tag == Const.goldTag) {        //碰到钻石
            otherCollider.node.active = false;

            let parent = otherCollider.node.parent;
            let basket: BasketLogic = parent.getComponent(BasketLogic);
            basket.showPrizeEffect();

            let random = Math.random();
            let score = 0;
            if (random < 0.3) {
                score = 2;
            }
            else {
                score = 1;
            }
            Global.addGold(score);

            cc.director.emit("addScore", {
                score: score,
                pos: parent.getPosition(),
                dir: parent.scaleX,
                type: 2
            })

        }
        else if (otherCollider.tag == Const.boomRangeTag) {     //炸弹区域
            let selfNode = selfCollider.node;
            let otherNode = otherCollider.node;
            let padX = selfNode.x - otherNode.x;
            let padY = selfNode.y - otherNode.y;

            this.setMoveInfo(padX * 6, 500);
        }
        else if (otherCollider.tag == Const.noBasketbodyTag) {     //篮框中心(取消碰撞检测)                   
            if (this.node.group != "ghost") {
                this.changMyGroup();
            }
        }
    }

}
