import { Global } from "../Global";
import { Const } from "../Const";
import { Message } from "../Message";

const { ccclass, property } = cc._decorator;

@ccclass
export default class Boom extends cc.Component {

    @property(cc.SpriteFrame)
    private boomSprite = null;

    private effect: sp.Skeleton;
    private mySprite: cc.Sprite;
    private boomRange: cc.Node;
    private physics: cc.RigidBody;  //炸弹的物理引擎
    private boomTimer = 0;

    private boomEnter = false;

    onLoad() {
        this.effect = this.node.getChildByName("effect").getComponent(sp.Skeleton);
        this.mySprite = this.node.getComponent(cc.Sprite);
        this.boomRange = this.node.getChildByName("boomRange");

        this.physics = this.getComponent(cc.RigidBody);
        this.physics.enabledContactListener = true;
    }

    onEnable() {
        this.effect.animation = "await4";
        this.mySprite.spriteFrame = this.boomSprite;
    }

    onDisable() {
        // this.physics.gravityScale = 0;
        // this.physics.linearVelocity = cc.v2(0, 0);
        // this.boomRange.active = false;
        // this.boomTimer = 0;   
        
        if(this.boomEnter){
            cc.director.emit("gameOver");
        }
    }

    public openTimer() {
        this.schedule(this.checkStop, 1);
    }

    public backHome() {
        this.unscheduleAllCallbacks()

        this.effect.animation = "await6";
        this.effect.loop = false;
        this.boomRange.active = true;
        this.mySprite.spriteFrame = null;

        this.scheduleOnce(() => {
            // this.node.parent = Global.boomParent;                
            // this.node.active = false; 
            if(Message.isOver){            
                return;
            }        
            Message.sendIsOver();
            this.destroy();
        }, 0.4)
    }

    private checkStop() {
        let pos = this.physics.linearVelocity;
        if (pos.x <= 20 && pos.y <= 20 && this.boomTimer >= 3) {
            this.unschedule(this.checkStop);

            if(!Message.isOver){ 
                Global.playMusic("boom_timer");
            }            

            this.scheduleOnce(this.backHome, 3);
        }
        else if (pos.x <= 20 && pos.y <= 20) {
            this.boomTimer++
        }
    }

    private setMoveInfo(x, y) {
        this.physics.linearVelocity = cc.v2(x, y);
    }


    private onBeginContact(contact, selfCollider, otherCollider) {
        if (otherCollider.tag == Const.BottomWallTag) {
            this.destroy();
        }
        else if (otherCollider.tag == Const.elasticityTag) {        //碰到弹力块
            let selfNode = selfCollider.node;
            let otherNode = otherCollider.node;
            let padX = selfNode.x - otherNode.x;
            let padY = selfNode.y - otherNode.y;

            this.setMoveInfo(padX * 10, padY * 10);

            otherCollider.node.runAction(cc.sequence(
                cc.scaleTo(0.1, 0.8, 0.8),
                cc.scaleTo(0.1, 1, 1)
            ))
        }
        else if (otherCollider.tag == Const.basketCenterTag) {        //碰到篮框
            this.boomEnter = true;
            cc.director.emit("boomEnter")
        }
    }

}
