import { Global } from "../Global";

const {ccclass, property} = cc._decorator;

let self:GameScene
@ccclass
export default class GameScene extends cc.Component {
    
    private btn_restart:cc.Node;
    private btn_pause:cc.Node;

    private pauseNode:cc.Node;

    onLoad(){
        self = this;

        self.btn_restart = self.node.getChildByName("btn_restart");
        self.btn_pause = self.node.getChildByName("btn_pause");
        self.pauseNode = self.node.getChildByName("pauseNode");

        self.btn_restart.on(cc.Node.EventType.TOUCH_END, self.restart, self);
        self.btn_pause.on(cc.Node.EventType.TOUCH_END, self.pause, self);

        cc.director.on("gameStart", self.openAtive, self);
        cc.director.on("againGame", self.openAtive, self);
        cc.director.on("gameOver", self.closeAtive, self);
        cc.director.on("gameInit", self.closeAtive, self);
        cc.director.on("tipRemainPen", self.tipRemainPen, self);
        cc.director.on("IsVictory", self.tipRemainPen, self);
    }

    private openAtive(){
        self.setActive(true);
    }

    private closeAtive(){
        self.setActive(false);

        cc.director.emit("tipRemainPen", {open:false});
    }

    private tipRemainPen(data){
        self.unschedule(self.tipRemain);
        let info = data.getUserData()

        if(info){
            if(info.open){
                self.scheduleOnce(self.tipRemain, 5);
            }
        }                
    }

    private tipRemain(){
        Global.setToast("点击屏幕即可划出线条！")
    }

    private setActive(state:boolean){
        if(state){
            //self.btn_restart.active = true;
            self.btn_pause.active = true;
        }
        else{
            self.btn_restart.active = false;
            self.btn_pause.active = false;
        }
    }

    private restart(){                
        cc.director.emit("stopTimer"); 

        cc.director.emit("againGame", {
            isSave: false
        });                
    }

    private pause(){
        cc.director.pause();
        self.pauseNode.active = true;
    }

}
