import physicsNodeLogic from "./Line/physicsNodeLogic";
import BasketLogic from "./BasketLogic";
import { Global } from "../Global";
import BallConLogic from "./BallConLogic";
import TipMaskControl from "./TipMaskControl";
import GamePrefabPool from "./GamePrefabPool";
import { Const } from "../Const";
import Boom from "./Boom";
import { Message } from "../Message";

const { ccclass, property } = cc._decorator;

let self: PassControl
@ccclass
export default class PassControl extends cc.Component {
    @property(physicsNodeLogic)
    private physicsNodeLogic: physicsNodeLogic = null;

    @property(GamePrefabPool)
    private gamePrefabPool: GamePrefabPool = null;

    @property(TipMaskControl)
    private tipMaskControl: TipMaskControl = null;

    @property(cc.Node)
    private popOpenBox: cc.Node = null;

    private frameArrayNode: cc.Node;
    private tempframeArray = [];

    private tempframeTopArray = [];
    private basketTopArray: cc.Node;

    private ballArray: cc.Node;

    private childBoom = [];
    private boomArray: cc.Node;

    private childLump = [];
    private lumpArray: cc.Node;


    onLoad() {
        self = this;
        self.frameArrayNode = self.node.getChildByName("frameArray");
        self.tempframeArray = self.frameArrayNode.children;

        self.basketTopArray = self.node.getChildByName("basketTopArray");
        self.tempframeTopArray = self.basketTopArray.children;

        self.ballArray = self.node.getChildByName("ballArray");
        self.boomArray = self.node.getChildByName("boomArray");
        self.lumpArray = self.node.getChildByName("lumpArray");

        self.childBoom = self.boomArray.children;
        self.childLump = self.lumpArray.children;

        cc.director.on("gameStart", self.gameStart, self);
        cc.director.on("IsVictory", self.IsVictory, self);
        cc.director.on("gameOver", self.gameOver, self);
        cc.director.on("againGame", self.againGame, self);
        cc.director.on("gameInit", self.hallInit, self);
        cc.director.on("stopTimer", self.stopTimer, self);
        cc.director.on("readComplete", self.readComplete, self);
        cc.director.on("boomEnter", self.boomEnter, self);

        cc.director.on("tipMode", self.againGame, self);

        if (cc.sys.platform != cc.sys.WECHAT_GAME) {
            self.hallInit();
        }
    }

    private stopTimer() {
        self.unscheduleAllCallbacks();
    }

    private readComplete() {
        self.hallInit();
    }

    private hallInit() {
        self.clearTool();
        self.clearBaskt();
        self.physicsNodeLogic.clearDraw();

        let posX = -116;
        let posY = 0;

        let frame = self.getObj(Const.basketLeftID);

        frame.parent = self.frameArrayNode;
        frame.setPosition(posX, posY);

        let basketTop = self.getObj(Const.basketTopID);

        basketTop.parent = self.basketTopArray;
        basketTop.setPosition(posX + 30, posY - 36);

        let ts = frame.getComponent(BasketLogic);
        ts.setType(Const.basketLeftID);

        let ball = this.getObj(Const.ballID);

        ball.parent = self.ballArray;        
        ball.setPosition(255, 160);
    }

    private gameStart() {
        Global.fuhuoCount = 0;
    }

    private nextPass() {
        Message.init();

        Global.setNewPass();
        self.setHaveFrame();
        self.setNoFrame();
        self.setBallData();
        self.setBoomData();
        self.setLumpData();
    }

    private gameOver() {
        self.clearTool();
        self.clearBaskt();
        self.physicsNodeLogic.clearDraw();

        if(Global.gameType == Const.passGame){        
            if (!Global.isGameStart) {
                self.hallInit();
                Global.curPenCount = 1;
            }
            else {
                Global.playMusic("fail", 0.7);
            }
        }        
    }

    private againGame(data) {
        let info = data.getUserData();
        if(info){
            Global.resetData(info.isSave);
        }
        
        self.clearTool();
        self.clearBaskt();
        self.physicsNodeLogic.clearDraw();

        self.scheduleOnce(() => {
            self.nextPass();
        }, 0.3)
    }

    private boomEnter() {
        Message.isBoomEnter = true;
    }

    private IsVictory() {
        if (Message.isBoomEnter) {
            return;
        }

        self.scheduleOnce(() => {
            var allBall = self.ballArray.children;            
            for (let i = 0; i < allBall.length; i++) {
                if (allBall[i]) {
                    return;
                }
            }
            if (Message.isJudge) {
                console.error("你不要进来000");
                return;
            }

            self.scheduleOnce(() => {
                if (Message.isJudge) {
                    console.error("你不要进来111");
                    return;
                }

                Message.isJudge = true;
                self.clearTool();
                self.clearBaskt();
                self.physicsNodeLogic.clearDraw();

                if(Global.gameType == Const.passGame){
                    self.isOpenBox();
                    Global.playMusic("passSuc", 0.7);                
    
                    self.scheduleOnce(() => {
                        self.nextPass();
                    }, 0.3)
                }
                else{
                    if(Global.tipSwitch){                        
                        self.scheduleOnce(() => {
                            self.physicsNodeLogic.clearDraw();
                            self.clearTool();
                            self.clearBaskt();
                            cc.director.emit("tipOver");                            
                        }, 0.3)                        
                    }
                    else{
                        self.scheduleOnce(() => {
                            cc.director.emit("gameOver", {isNext:true});
                        }, 0.3)
                    }
                }
            }, 0.6)
        }, 0.2)
    }

    private isOpenBox() {
        if (Global.getConfig2) {
            if (Global.getConfig2.shareGoldSwtich != 0) {
                if (Math.random() < 0.20) {
                    self.popOpenBox.active = true;
                }
            }
        }
    }

    private clearTool() {
        let ballArray = self.ballArray.children;
        for (let i = 0; i < ballArray.length; i++) {
            let ball = ballArray[i];
            ball.destroy();
        }

        let boomArray = self.childBoom;
        for (let i = 0; i < boomArray.length; i++) {
            //let boom = boomArray[i].getComponent(Boom);
            let boom = boomArray[i];
            boom.destroy();
        }

        let lumpArray = self.childLump;
        for (let i = 0; i < lumpArray.length; i++) {
            let lump = lumpArray[i];
            lump.destroy();
        }
    }
    private clearBaskt() {
        let frameArray = self.tempframeArray;
        for (let i = 0; i < frameArray.length; i++) {
            let basketLogic = frameArray[i].getComponent(BasketLogic);
            basketLogic.backHome();
        }
        self.tempframeArray = [];

        let ballTopArray = self.tempframeTopArray;
        for (let i = 0; i < ballTopArray.length; i++) {
            ballTopArray[i].active = false;
            ballTopArray[i].parent = Global.basketTopParent;
        }
        self.tempframeTopArray = [];
    }

    /**
     * 
     * @param type 1.球  2.篮框下(左)  3.篮框下(右)  4. 篮框上  5.无篮框
     */
    private getObj(type) {
        let obj = self.gamePrefabPool.getObj(type);
        if (type == Const.basketLeftID || type == Const.basketRightID || type == Const.nobasketID) {
            self.tempframeArray.push(obj);
        }
        if (type == Const.basketTopID) {
            self.tempframeTopArray.push(obj);
        }

        return obj;
    }

    //设置关卡
    private setHaveFrame() {
        var posArray = Global.getFrameData();
        if (posArray) {
            for (let i = 0; i < posArray.length; i++) {
                if (posArray[i]) {
                    let posX = posArray[i][0];
                    let posY = posArray[i][1];

                    let frame;
                    let basketTop = self.getObj(Const.basketTopID);

                    if (posX == -255) {
                        frame = self.getObj(Const.basketLeftID);
                        basketTop.parent = self.basketTopArray;
                        basketTop.setPosition(posX + 30, posY - 36);

                        let ts = frame.getComponent(BasketLogic);
                        ts.setType(Const.basketLeftID);
                    }
                    else if (posX == 255) {
                        frame = self.getObj(Const.basketRightID);
                        basketTop.parent = self.basketTopArray;
                        basketTop.setPosition(posX - 30, posY - 36);

                        let ts = frame.getComponent(BasketLogic);
                        ts.setType(Const.basketRightID);
                    }

                    frame.parent = self.frameArrayNode;
                    frame.setPosition(posX, posY);
                }
            }
        }
    }

    private setNoFrame() {
        var posArray = Global.getNoFrameData();

        if (posArray) {
            for (let i = 0; i < posArray.length; i++) {
                if (posArray[i]) {
                    let posX = posArray[i].pos[0];
                    let posY = posArray[i].pos[1];

                    let frame = self.getObj(Const.nobasketID);
                    let basketTop = self.getObj(Const.basketTopID);

                    let dir = posArray[i].dir;

                    frame.parent = self.frameArrayNode;
                    frame.setPosition(posX, posY);
                    frame.rotation = dir;

                    let ts = frame.getComponent(BasketLogic);
                    ts.setType(Const.nobasketID);

                    basketTop.parent = self.basketTopArray;

                    if (dir == -45) {
                        basketTop.setPosition(posX - 27, posY + 27);
                    }
                    else if (dir == 45) {
                        basketTop.setPosition(posX + 27, posY + 27);
                    }
                    else if (dir == -30) {
                        basketTop.setPosition(posX - 23, posY + 32);
                    }
                    else if (dir == 30) {
                        basketTop.setPosition(posX + 18, posY + 35);
                    }
                    else {
                        basketTop.setPosition(posX - 2, posY + 38);
                    }

                    basketTop.rotation = dir;
                }
            }
        }
    }

    private setBallData() {
        var ballArray = Global.getBallData();

        if (ballArray) {
            for (let i = 0; i < ballArray.length; i++) {
                let data = ballArray[i];
                if (data) {
                    let pos = data.pos;

                    let appearY = data.pos[1] - (720 - Global.sceneWight);

                    let ball = this.getObj(Const.ballID);                    

                    ball.parent = self.ballArray;
                    ball.setPosition(pos[0], appearY);

                    self.tipMaskControl.setTip(ball, data, Const.ballID);
                }
            }
        }
    }

    private setBoomData() {
        var boomArray = Global.getBoomData();
        if (boomArray) {
            for (let i = 0; i < boomArray.length; i++) {
                let data = boomArray[i];
                if (data) {
                    let pos = data.pos;
                    let appearY = data.pos[1] - (720 - Global.sceneWight);

                    let boom = this.getObj(Const.boomID);

                    boom.parent = self.boomArray;

                    boom.setPosition(pos[0], appearY);
                    self.tipMaskControl.setTip(boom, data, Const.boomID);
                }
            }
        }
    }

    private setLumpData() {
        var lumpArray = Global.getLumpData();

        if (lumpArray) {
            for (let i = 0; i < lumpArray.length; i++) {
                let data = lumpArray[i];
                if (data) {
                    let pos = data.pos;

                    let lump = this.getObj(Const.lumpID);

                    lump.parent = self.lumpArray;
                    lump.setPosition(pos[0], pos[1]);
                }
            }
        }
    }
}
