import Toast from "./Toast";
import { wxApply } from "./wxGame/wxApply";
import Rank from "./Rank";
import PenControl from "./GameScene/PenControl";
import PopEffect from "./GameScene/PopEffect";
import GameScore from "./GameScore";
import EnterPassTipUI from "./AloneScene/EnterPassTipUI";
import { Const } from "./Const";
import AloneGameUI from "./AloneScene/AloneGameUI";

export namespace Global {
    export let curPenCount = 1;        //蜡笔总数量 

    export let myuid;
    export let openid;

    export let curBanner;   //banner广告

    export let sdkQQ;

    export let curScore = 0;
    export let USER_INFO = {
        nickname: "",
        maxScore: 0,
        gold: 200,
        shareGoldCount: 0,  //分享得金币次数
        curDay: 0,          //当前日子
        curTimer: 0,        //当前时间轴
        selectBall: 0,
        selectLine: 0,
        curLineColor: "#C4C4C4",
        newPlayer: true,
        alonePassNum: 1,
        maxAlonePassNum: 0,
        isOpenNewGuide: true
    };
    export let ownBall = [0];
    export let ownLine = [0];

    //是否开启提示模式
    export let tipSwitch = false;

    //当前游戏模式
    export let gameType = Const.passGame;

    //运营配置信息
    export let getConfig1;

    //技术配置信息
    export let getConfig2;

    //是否播放音乐
    export let isPlayMusic = true;

    //我的钻石
    export let myGoldCount: cc.Label;

    //手机类型
    export let motorType;

    //适配比例大小
    export let stageScale = 1;
    export let sceneWight;
    export let sceneHeight;

    //是否游戏开始
    export let isGameStart = false;

    //游戏提示
    export let toast: Toast;

    //游戏加分
    export let gameScore: GameScore;

    //游戏闯关
    export let aloneUI: cc.Node;

    //当前关卡
    export let aloneGameUI: AloneGameUI;

    //游戏蜡笔
    export let penArray: PenControl;

    //排行榜
    ///|export let rankUI: Rank;

    //加分特效
    export let popEffect: PopEffect;

    //闯关
    export let enterPassTipUI: EnterPassTipUI;

    //对象池
    export let basketTopParent: cc.Node;
    export let basketParentLeft: cc.Node;
    export let basketParentRight: cc.Node;
    export let nobasketBall: cc.Node;
    export let ballParent: cc.Node;
    export let boomParent: cc.Node;
    export let lumpParent: cc.Node;

    //复活次数
    export let fuhuoCount = 0;

    //关卡数据
    export let passData = [];
    export let curPassData;
    export let curPass = 1;
    export let curMaxPass = 1;
    export let curPassIndex = 1;

    //道具数据
    export let toolData = [];

    //单人闯关表
    export let alonePassData = [];
    export let aloneToolData = [];

    //使用蜡笔
    export function usePen() {
        if (curPenCount > 0) {
            curPenCount--;
            setPen(1);

            if (curPenCount <= 0) {
                cc.director.emit("openOverTimer");
                cc.director.emit("tipRemainPen", { open: false });
            }
            else {
                cc.director.emit("tipRemainPen", { open: true });
            }

            return true;
        }
        else {
            setToast("蜡笔不足");
            return false;
        }
    }

    //复原新的关卡
    export function recoverNewPass() {
        curPenCount = 3;
        setPen(2);
    }

    /**
     * 
     * @param type 1.使用 2.复原
     */
    export function setPen(type) {
        penArray.setPen(type);
    }

    export function addSocre(score) {
        if (Global.gameType == Const.passGame) {
            if (curScore == 0) {
                cc.director.emit("gameStart");
                Global.isGameStart = true;
            }

            curScore = curScore + score;
            updateScore();
        }

        if (score == 1) {
            popEffect.showEffect(1);
        }
        else if (score == 2) {
            popEffect.showEffect(2);
        }
    }

    export function setToast(str) {
        toast.node.active = true;
        toast.setLable(str);
    }

    //数据加载
    export function initData() {
        cc.loader.loadRes(`Json/PassData.json`, function (err, data) {         //读取角色表
            if (err) {
                console.log("读取关卡表json表失败", err);
            } else {
                passData = data;
                curPassData = data[curPassIndex];

                let passRange = data[curPassIndex].passRange;
                curMaxPass = passRange[1];
            }
        })

        cc.loader.loadRes(`Json/ToolData.json`, function (err, data) {
            if (err) {
                console.log("读取道具表json表失败", err);
            } else {
                toolData = data;
            }
        })


        cc.loader.loadRes(`Json/AlonePassData.json`, function (err, data) {
            if (err) {
                console.log("读取单人关卡表json表失败", err);
            } else {
                alonePassData = data;
            }
        })

        cc.loader.loadRes(`Json/AloneToolData.json`, function (err, data) {
            if (err) {
                console.log("读取单人道具表json表失败", err);
            } else {
                aloneToolData = data;
            }
        })

        if (cc.sys.platform == cc.sys.WECHAT_GAME) {
            //初始化sdk
            sdk.init({
                debug: true,         //.是否开启调试              
            }, (res) => {
                console.log('sdk初始化结果：', res)
                var yunYing = sdk.getConfig1();
                Global.getConfig1 = yunYing;

                var jiShu = sdk.getConfig2();
                Global.getConfig2 = jiShu;

                cc.director.emit("initSdkComplete");

                console.log("运营配置信息", yunYing);
                console.log("技术配置", jiShu);
                console.log("sdk初始化完成");
            })
        }
    }

    //重置数据
    export function resetData(isSave) {
        if (Global.gameType == Const.passGame) {
            if (!isSave) {
                Global.curScore = 0;
                curPass = 1;
                curMaxPass = 1;
                curPassIndex = 1;
                curPenCount = 1;
                Global.fuhuoCount = 0;                            
            }

            setPen(2);
            updateScore();

            let passRange = passData[curPassIndex].passRange;
            curMaxPass = passRange[1];
        }
        else {

        }
    }

    //关卡信息设置
    export function setNewPass() {
        if (Global.gameType == Const.passGame) {
            curPass++;

            //重置数据
            recoverNewPass();

            if (curPass > curMaxPass) {
                curPassIndex++;
            }

            curPassData = passData[curPassIndex];
            let passRange = passData[curPassIndex].passRange;
            curMaxPass = passRange[1];
        }
        else {
            Global.isGameStart = true;

            //重置数据
            recoverNewPass();

            Global.aloneGameUI.updatePass();

            curPassData = alonePassData[curPassIndex];

            //curPassIndex++;
        }

        console.log("当前关卡信息", Global.curPassIndex);
    }

    export function getFrameData() {
        let haveFrameRandom = curPassData.haveFrameRandom;
        if (haveFrameRandom == 0) {
            return;
        }

        let random = Math.floor(Math.random() * (haveFrameRandom[1] - haveFrameRandom[0] + 1) + haveFrameRandom[0]);

        let frameData = [];
        let haveFramePos = curPassData.haveFramePos.slice(0);
        let tempHaveFramePos = copyArray(haveFramePos, random);

        for (let i = 0; i < random; i++) {
            frameData.push(tempHaveFramePos[i][0]);
        }
        return frameData;
    }

    export function getNoFrameData() {
        if (curPassData.noFrameRandom == 0) {
            return;
        }

        let haveFrameRandom = curPassData.noFrameRandom;
        let random = Math.floor(Math.random() * (haveFrameRandom[1] - haveFrameRandom[0] + 1) + haveFrameRandom[0]);

        let frameData = [];
        let noHaveFramePos = curPassData.noFramePos.slice(0);

        let noHaveFrameDic;

        let tempNoHaveFramePos = copyArray(noHaveFramePos, random);

        for (let i = 0; i < random; i++) {
            if (curPassData.noFrameDic.length) {
                noHaveFrameDic = curPassData.noFrameDic[i];
            }
            else {
                noHaveFrameDic = curPassData.noFrameDic;
            }

            frameData.push({ pos: tempNoHaveFramePos[i][0], dir: noHaveFrameDic });
        }
        return frameData;
    }

    export function copyArray(array, count) {
        let tempArray = [];
        for (let i = 0; i < count; i++) {
            let index = Math.floor(Math.random() * (array.length - 1));
            tempArray.push(array.splice(index, 1))
        }
        return tempArray;
    }

    export function getBallData() {
        let maxBallCount = curPassData.ballTool;
        if (maxBallCount == 0) {
            return;
        }

        let ballTool;
        if (maxBallCount.length) {
            ballTool = maxBallCount[0];
        }
        else {
            ballTool = Math.floor(Math.random() * maxBallCount + 1);
        }

        let toolId = curPassData.ballID.slice(0);
        let tempArray = copyArray(toolId, ballTool);
        let returnToolArray = [];

        for (let i = 0; i < ballTool; i++) {
            var toolData = getToolData(tempArray[i]);
            returnToolArray.push(toolData);
        }
        //console.log("球", returnToolArray);
        return returnToolArray;
    }

    export function getBoomData() {
        let maxBallCount = curPassData.boomTool;
        if (maxBallCount == 0) {
            return;
        }

        let boomTool;
        if (maxBallCount.length) {
            boomTool = maxBallCount[0];
        }
        else {
            boomTool = Math.floor(Math.random() * maxBallCount + 1);
        }

        let toolId = curPassData.boomID.slice(0);
        let tempArray = copyArray(toolId, boomTool);
        let returnToolArray = [];

        for (let i = 0; i < boomTool; i++) {
            var toolData = getToolData(tempArray[i]);
            returnToolArray.push(toolData);
        }
        return returnToolArray;
    }

    export function getLumpData() {
        let maxBallCount = curPassData.lumpTool;
        if (maxBallCount == 0) {
            return;
        }

        let lumpTool;
        if (maxBallCount.length) {
            lumpTool = maxBallCount[0];
        }
        else {
            lumpTool = Math.floor(Math.random() * maxBallCount + 1);
        }

        let toolId = curPassData.lumpID.slice(0);
        let tempArray = copyArray(toolId, lumpTool);
        let returnToolArray = [];

        for (let i = 0; i < lumpTool; i++) {
            var toolData = getToolData(tempArray[i]);
            returnToolArray.push(toolData);
        }
        //console.log("弹力块", returnToolArray);
        return returnToolArray;
    }

    export function getToolData(toolId) {
        if (Global.gameType == Const.passGame) {
            for (let i = 0; i < toolData.length; i++) {
                if (toolData[i].id == toolId) {
                    return toolData[i];
                }
            }
        }
        else {
            for (let i = 0; i < aloneToolData.length; i++) {
                if (aloneToolData[i].id == toolId) {
                    return aloneToolData[i];
                }
            }
        }
    }


    //----------------------------------------本地---------------------------------------------

    //保存
    export function saveNativeData() {
        wxApply.setStorage("myData", Global.USER_INFO)
    }

    //读取
    export function getNativeData() {
        wxApply.getStorage("myData")
    }


    //保存商店
    export function saveShopData() {
        wxApply.setStorage("shopBall", Global.ownBall);
        wxApply.setStorage("shopLine", Global.ownLine);
    }

    //读取商店
    export function getShopData() {
        wxApply.getStorage("shopBall");
        wxApply.getStorage("shopLine");
    }

    //------------------------------全局功能----------------------------------
    export function playMusic(musicName: string, volume = 1) {
        if (isPlayMusic) {
            var audioName: string = "sound/" + musicName;
            cc.loader.loadRes(audioName, function (err, assets) {
                cc.audioEngine.play(assets, false, volume);
            })
        }
    }

    export function stopAllMusic() {
        cc.audioEngine.stopAll();
    }

    //更新关卡信息
    export function updateAlone(passCount) {
        if (Global.USER_INFO.alonePassNum < passCount) {
            Global.USER_INFO.alonePassNum = passCount;
            Global.USER_INFO.maxAlonePassNum = passCount - 1;
            saveNativeData();
        }
    }

    //更新我的分数
    export function updateScore() {
        //gameScore.string = `X${curScore}分`;
        gameScore.updateScore();
    }

    //更新我的钻石
    export function updateGold() {
        myGoldCount.string = Global.USER_INFO.gold + "";
        saveNativeData();
    }

    //添加钻石
    export function addGold(count) {
        Global.USER_INFO.gold += count;
        playMusic("gold");
        updateGold();
    }

    //消费
    export function useGold(count) {
        if (Global.USER_INFO.gold >= count) {
            Global.USER_INFO.gold = Global.USER_INFO.gold - count;            
            updateGold();
            return true;
        }
        else {
            setToast("钻石不足");
            return false;
        }
    }

    //获取奖励等级对应的钻石
    export function getGradeCount(grade) {
        switch (grade) {
            case 1:
                return 20;
            case 2:
                return 40;
            case 3:
                return 60;
            case 4:
                return 80;
            case 5:
                return 100;
        }
    }

    //获取奖励等级对应的时间
    export function getGradeTimer(grade) {
        switch (grade) {
            case 1:
                return 20;
            case 2:
                return 40;
            case 3:
                return 60;
            case 4:
                return 80;
            case 5:
                return 100;
        }
    }

    //是否打开奖励开关
    export function isOpenPrize() {
        // let judgeTime = getGradeTimer(USER_INFO.prizeGrade);
        // if (USER_INFO.onLineTimer > judgeTime) {
        //     USER_INFO.prizeGrade++;

        //     if (USER_INFO.prizeGrade > 5) {
        //         USER_INFO.prizeGrade = 5;
        //     }

        //     return true;
        // }
        // else {
        //     return false;
        // }
    }

    //判断分享次数是否达到今日上限
    export function judgeShareMax() {
        USER_INFO.shareGoldCount++;

        if (USER_INFO.shareGoldCount >= 5) {
            var myDate = new Date();
            USER_INFO.curDay = myDate.getDate();    //保存日子

            cc.director.emit("closeShareGold");
        }
    }

    //检查今天的分享是否达到今日上限（登录时直接判断）
    export function judgeShareDayMax() {
        var myDate = new Date();
        console.log("今天的分享限制", myDate.getDate(), USER_INFO.curDay, USER_INFO.shareGoldCount);

        if (USER_INFO.curDay == myDate.getDate()) {
            if (USER_INFO.shareGoldCount >= 5) {
                cc.director.emit("closeShareGold");
            }
        }
        else {
            console.log("重置分享次数");
            USER_INFO.shareGoldCount = 0;
        }
    }

    //比较在线时间
    export function judgeOnTimer() {
        let myDate = new Date();
        let pad = myDate.getTime() - USER_INFO.curTimer;

        if (pad > 10800000) {
            return true;
        }
        else {
            return false;
        }
    }

    //广告位的创建
    export function creatBanner(id) {
        ///| if (cc.sys.platform == cc.sys.WECHAT_GAME) {
        ///|     if(Global.getConfig1.bannerAd == 1){
        ///|         var myWidth = tt.getSystemInfoSync().screenWidth;
        ///|         var myheigh = tt.getSystemInfoSync().screenHeight;
        ///|         let bannerAd = tt.createBannerAd({
        ///|             adUnitId: id,
        ///|             style: {
        ///|                 left: 0,
        ///|                 top: myheigh - 120,
        ///|                 width: myWidth,
        ///|             }
        ///|         })
        ///|         bannerAd.onResize(res => {
        ///|             bannerAd.style.top = myheigh - res.height;
        ///|         })
        ///|         curBanner = bannerAd;
        ///|         bannerAd.show()
        ///|     }
        ///| }
    }

    //广告位的隐藏
    export function hideBanner() {
        ///|if (cc.sys.platform == cc.sys.WECHAT_GAME) {
            if(Global.getConfig1.bannerAd == 1){
                ///|curBanner.hide();
            }            
        ///|}        
    }

    //视频激励广告
    export function createRewardedVideoAd(adUnitId, callback?: Function, err?: Function) {
        ///| if (Global.getConfig1.videoAd != 1) {
        ///|     return;
        ///| }

        ///| let videoAd;
        ///| videoAd = tt.createRewardedVideoAd({
        ///|     adUnitId: adUnitId
        ///| })

        ///| videoAd.load()
        ///|     .then(() => {
        ///|         videoAd.show();
        ///|         // cc.audioEngine.stop(Global.bgm);
        ///|     })
        ///|     .catch(err => {
        ///|         videoAd.load()
        ///|             .then(() => {
        ///|                 videoAd.show();
        ///|                 // cc.audioEngine.stop(Global.bgm);
        ///|             }
        ///|         )
        ///|     })

        ///| videoAd.onClose(videoClose);

        ///| function videoClose(res) {
        ///|     videoAd.offClose(videoClose);
        ///|     if (res && res.isEnded || res === undefined) {
        ///|         callback();
        ///|     }
        ///|     else {
        ///|         err();
        ///|     }
        ///| }
    }
}
