import Toast from "./Toast";
import { Global } from "./Global";
import GameTip from "./GameTip";
import { wxApply } from "./wxGame/wxApply";
import Rank from "./Rank";
import PenControl from "./GameScene/PenControl";
import PopEffect from "./GameScene/PopEffect";
import { Message } from "./Message";
import GameScore from "./GameScore";
import EnterPassTipUI from "./AloneScene/EnterPassTipUI";
import { Const } from "./Const";
import AloneGameUI from "./AloneScene/AloneGameUI";

const { ccclass, property } = cc._decorator;

let self: SceneControl
@ccclass
export default class SceneControl extends cc.Component {
    @property(Toast)
    private toast = null;

    @property(cc.Node)
    private physicsArray = null;

    @property(cc.Node)
    private bg = null;

    @property(cc.Node)
    private touchStop = null;

    @property(GameTip)
    private gameTip: GameTip = null;

    @property(PopEffect)
    private popEffect: PopEffect = null;

    @property(cc.Node)
    private gameOverUI: cc.Node = null;

    @property(cc.Node)
    private physicsNode: cc.Node = null;

    @property(cc.Node)
    private gameHall: cc.Node = null;

    @property(EnterPassTipUI)
    private enterPassTipUI = null;

    ///|@property(Rank)
    ///|private rankUI: Rank = null;

    @property(GameScore)
    private gameScore: GameScore = null;

    @property(AloneGameUI)
    private aloneGameUI: AloneGameUI = null;

    @property(cc.Label)
    private myGoldCount: cc.Label = null;

    @property(cc.Node)
    private aloneUI:cc.Node = null;

    //游戏里
    private gameScene: cc.Node;

    @property(cc.Node)
    private penNode: cc.Node = null;

    private penArray: PenControl = null;

    onLoad() {
        self = this;
        self.gameScene = self.node.getChildByName("GameScene");
        //self.penNode = self.gameScene.getChildByName("pen");
        self.penArray = self.penNode.getComponent(PenControl);

        Global.toast = self.toast;
        Global.gameScore = self.gameScore;
        Global.myGoldCount = self.myGoldCount;
        Global.penArray = self.penArray;
        ///|Global.rankUI = self.rankUI;
        Global.popEffect = self.popEffect;
        Global.enterPassTipUI = self.enterPassTipUI;
        Global.aloneGameUI = self.aloneGameUI;
        Global.aloneUI = self.aloneUI;

        Global.initData();

        // self.scheduleOnce(() => {
        //     // Global.getFrameData();
        //     // Global.getBallData();
        // }, 0.5)

        self.addEvent();

        cc.director.getPhysicsManager().enabled = true;
        cc.director.getPhysicsManager().gravity = cc.v2(0, -980);

        // cc.director.getPhysicsManager().debugDrawFlags = cc.PhysicsManager.DrawBits.e_aabbBit |
        //     cc.PhysicsManager.DrawBits.e_pairBit |
        //     cc.PhysicsManager.DrawBits.e_centerOfMassBit |
        //     cc.PhysicsManager.DrawBits.e_jointBit |
        //     cc.PhysicsManager.DrawBits.e_shapeBit
        //     ;

        cc.director.setDisplayStats(false);

        self.onStage();
    }

    start() {
        Global.getNativeData();
        Global.getShopData();
        
        //Global.saveNativeData();
    }

    private addEvent() {
        cc.director.on("gameStart", self.gameStart, self);
        cc.director.on("gameOver", self.gameOver, self);
        cc.director.on("gameInit", self.gameInit, self);
        cc.director.on("againGame", self.againGame, self);
        cc.director.on("tipMode", self.tipMode, self);
        cc.director.on("tipOver", self.tipOver, self);

        wxApply.onHide(self.onHide);
    }

    private gameStart() {
        if (Global.gameType == Const.passGame) {
            self.gameScore.node.active = true;
        }
        else {
            self.aloneGameUI.node.active = true;
        }


        self.penNode.active = true;
    }

    private againGame() {
        ///|Global.rankUI.closeMy();
        self.gameOverUI.active = false;
    }

    private gameOver(data) {
        if (Global.gameType == Const.passGame) {
            if (!Global.isGameStart) {
                this.gameTip.showMy();
            }

            if (Global.USER_INFO.maxScore < Global.curScore) {
                Global.USER_INFO.maxScore = Global.curScore;
                wxApply.setUserCloudStorage(Global.curScore);
            }

            if (Global.isGameStart) {
                //打开结束界面
                self.gameOverUI.active = true;
            }
        }
        else {
            let isNext = false;

            if (data.getUserData()) {
                isNext = data.getUserData().isNext;
            }

            if (!isNext) {
                Global.isGameStart = false;                                
            }

            if(Global.tipSwitch){
                Global.tipSwitch = false;
                self.tipOver();
            }                        
            self.gameOverUI.active = true;
        }
    }

    private tipOver() {
        //原来的颜色#FFE96E
        //self.bg.color = cc.hexToColor("#FFE96E");
        self.touchStop.active = false;        
    }

    private onHide() {
        Global.stopAllMusic();
    }

    private gameInit() {
        self.gameTip.showMy();
        self.gameHall.active = true;

        self.gameScore.node.active = false;
        self.aloneGameUI.node.active = false;
        self.penNode.active = false;

        Message.init();
        Global.stopAllMusic();
        Global.resetData(false);                  

        self.physicsNode.active = true;
    }

    private tipMode() {
        //原来的颜色#FFE96E
        //self.bg.color = cc.hexToColor("#FFFFFF");
        self.touchStop.active = true;
    }

    private onStage() {
        Global.sceneWight = self.node.width;
        Global.sceneHeight = self.node.height;
        console.log("屏幕宽高", Global.sceneWight, Global.sceneHeight);

        if (self.node.width < 720) {
            let stageScale = self.node.width / 720;
            Global.stageScale = Math.floor(stageScale);

            console.log("适配调整值", stageScale);

            self.gameHall.setScale(stageScale);

            //self.gameScene.setScale(stageScale);
            self.gameScene.width = 720;
            self.gameScene.height = 1280;           

            ///|self.rankUI.node.setScale(stageScale);

            //self.physicsNode.setScale(stageScale);               
        }
        if (cc.sys.platform == cc.sys.WECHAT_GAME) {
            if (sharedCanvas) {
                sharedCanvas.width = cc.game.canvas.width * 2;
                sharedCanvas.height = cc.game.canvas.height * 2;
            }
        }
    }
}
